# -*- coding: utf-8 -*-
# Copyright 2017 Budapest University of Technology and Economics (BME IK)
#
# This file is part of CIRCLE Cloud.
#
# CIRCLE is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# CIRCLE is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along
# with CIRCLE.  If not, see <http://www.gnu.org/licenses/>.


import cmd
from parsers import VMParser, NetworkParser, TemplateParser


def parser(parser_class):
    def top_wrapper(func):
        def wrapper(self, arg):
            try:
                args = parser_class().parse_args(arg.split())
                return func(self, args)
            except SystemExit:
                pass
        return wrapper
    return top_wrapper


class OCCIShell(cmd.Cmd):
    intro = ('Wellcome to the CIRCLE cloud shell.'
             ' Type help or ? to list commands.\n')

    prompt = '(CIRCLE) '

    def do_print(self, arg):
        print(arg)

    @parser(VMParser)
    def do_vm(self, arg):
        print("vm %s" % unicode(arg))

    @parser(TemplateParser)
    def do_template(self, arg):
        print("template %s" % unicode(arg))

    @parser(NetworkParser)
    def do_network(self, arg):
        print("network %s" % unicode(arg))

    def do_exit(self, arg):
        exit()

    def do_quit(self, arg):
        exit()

    def precmd(self, line):
        if line == "EOF":  # Ctrl + D
            print("")
            exit()
        return line
